/*
 * Connection tracking state replication for Netfilter
 *
 * (C) 2002-2003, KOVACS Krisztian <hidden@sch.bme.hu>
 * (C) 2003-2004, Harald Welte <laforge@netfilter.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * $Id: ct_sync.h,v 1.19 2004/08/30 11:05:46 hidden Exp $
 *
 */
#ifndef _CT_SYNC_H
#define _CT_SYNC_H

#if 0
#include <linux/config.h>
#include <linux/version.h>

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tqueue.h>
#include <linux/wait.h>

#include <asm/semaphore.h>
#include <linux/if_ether.h>
#endif

#include <linux/list.h>
#include <linux/types.h>
#include <linux/if_ether.h>
#include <linux/netfilter_ipv4/ip_conntrack.h>

#if 0
#include <linux/netfilter_ipv4/ip_conntrack_protocol.h>
#include <linux/netfilter_ipv4/ip_conntrack_helper.h>
#include <linux/netfilter_ipv4/ip_conntrack_core.h>
#include <linux/netfilter_ipv4/ip_nat.h>
#include <linux/netfilter_ipv4/ip_nat_helper.h>
#endif

#define CTSYNC_DEBUG 0

/* debugging macros */
#if (CTSYNC_DEBUG > 0)
#define CT_SYNC_DEBUG(x, args...) printk (KERN_DEBUG "ct_sync: %s " x, __FUNCTION__ , ## args)
#define CT_SYNC_ASSERT(x) do {						\
	if (!(x)) 							\
	printk (KERN_ERR "ct_sync: %s:%i(%s) assertion failed.\n", 	\
		__FILE__, __LINE__, __FUNCTION__);			\
} while (0)
#else
#define CT_SYNC_DEBUG(x, args...)
#define CT_SYNC_ASSERT(x, args...)
#endif

#if (CTSYNC_DEBUG >= 2)
#define CT_SYNC_ENTER()		CT_SYNC_DEBUG("enter\n") 
#define CT_SYNC_LEAVE()		CT_SYNC_DEBUG("leave\n")
#else
#define CT_SYNC_ENTER()
#define CT_SYNC_LEAVE()
#endif

#define CT_SYNC_ERR(x, args...) printk(KERN_ERR "ct_sync: %s " x, __FUNCTION__ , ## args)
#define CT_SYNC_INFO(x, args...) printk(KERN_INFO "ct_sync: %s " x, __FUNCTION__ , ## args)

/* predefined sizes */
#define CT_SYNC_CTHELPERSIZE	16	/* maximal length of ct helper name */
#define CT_SYNC_NATHELPERSIZE	16	/* maximal length of NAT helper name */
#define CT_SYNC_PAYLOADSIZE	ETH_DATA_LEN - 20 - 8 - sizeof(struct ct_sync_pkthdr)	/* maximal useful size of sync packets*/

#define CTMSG_SIZEOF(x)		(sizeof(x)+sizeof(struct ct_sync_msghdr))

/* packet header */
struct ct_sync_pkthdr
{
	__u8		version;		/* version number */
	__u8		pkttype;		/* type of packet: sync/NACK */
	__u16		pktseq;			/* packet sequence number */
	__u8		node_id;		/* id of the sender node */
	__u8		count;			/* number of msgs in packet */
	__u16		minseq;			/* min seqno master has */
	__u32		__reserved;		/* placeholder :) */
};

/* packet types */
enum ct_sync_pkttype_t
{
	CT_SYNC_PKT_UNKNOWN,			/* not defined, illegal */
	CT_SYNC_PKT_MASTER_ANNOUNCE,
	CT_SYNC_PKT_SLAVE_INITSYNC,
	CT_SYNC_PKT_SYNC,			/* normal sync packet */
	CT_SYNC_PKT_NACK,			/* negative ack packet */
	CT_SYNC_PKT_INITSYNC_FOLLOWS,		/* a full re-sync follows */
};

/* CT_SYNC_PKT_MASTER_ANNOUNCE */
struct ct_sync_master_announce
{
	__u32	upsince;			/* master up since */
	__u8	flags;				/* flags */
};

/* CT_SYNC_PKT_SLAVE_INITSYNC */
struct ct_sync_slave_initsync
{
	__u32	master;
};

/* CT_SYNC_PKT_NACK doesn't have any payload */

/* definitions for CT_SYNC_PKT_SYNC */
struct ct_sync_pkt
{
	struct ct_sync_pkthdr	hdr;
	__u8			data[CT_SYNC_PAYLOADSIZE];
};

/* CT_SYNC_PKT_SYNC: message header */
struct ct_sync_msghdr
{
	__u8		resource;	/* which table? */
	__u8		type;		/* type of message: ct_sync_msgtype_t */
	__u16		len;		/* length of message */
};

/* CT_SYNC_PKT_SYNC: message types */
enum ct_sync_msgtype_t
{
	CT_SYNC_MSG_UPDATE = 0,			/* update conntrack entry 
						   (or create if not exists) */
	CT_SYNC_MSG_DELETE,			/* delete conntrack entry */
};

/* CT_SYNC_PKT_SYNC: ressources */
enum ct_sync_resource_t
{
	CT_SYNC_RES_CONNTRACK = 0,
	CT_SYNC_RES_EXPECT,
};

/* linearized conntrack data */
struct ct_sync_conntrack
{
	struct ip_conntrack_tuple	orig, reply;	/* tuples */
	__u32				status;		/* status */
	__u32				expires;	/* timeout */

	/* conntrack helper */
	char				helper[CT_SYNC_CTHELPERSIZE];	/* conntrack helper name */
	union ip_conntrack_help		help;			/* conntrack helper specific info */

	unsigned int			expecting;	/* number of siblings */

	struct ip_conntrack_tuple	master_ct_tuple; /* master ct tuple */

	/* protocol */
	union ip_conntrack_proto	proto;

	/* NAT */
	#ifdef CONFIG_IP_NF_NAT_NEEDED
	int				nat_initialized;
	__u8				nat_num_manips;
	struct ip_nat_info_manip 	nat_manips[IP_NAT_MAX_MANIPS];
	char				nat_helper[CT_SYNC_NATHELPERSIZE];
	union ip_conntrack_nat_help	nat_help;
	struct ip_nat_seq		nat_seq[IP_CT_DIR_MAX];
	#if defined(CONFIG_IP_NF_TARGET_MASQUERADE) || defined(CONFIG_IP_NF_TARGET_MASQUERADE_MODULE)
	char				nat_masq_iface[IFNAMSIZ];
	#endif /* MASQUERADE */
	#endif /* NAT */

	/* mark */
	#ifdef CONFIG_IP_NF_CONNTRACK_MARK
	__u32 				mark;
	#endif
};

/* linearized expectation data */
struct ct_sync_expect
{
	struct ip_conntrack_tuple	expectant;	/* tuple of master conntrack */
	struct ip_conntrack_tuple	sibling;	/* tuple of sibling conntrack, if already arrived */

	struct ip_conntrack_tuple	tuple, mask;	/* expectation tuple and mask */
	__u32				seq;		/* sequence number */
	union ip_conntrack_expect_proto	proto;		/* protocol specific info */
	union ip_conntrack_expect_help	help;		/* expectation helper specific info */
};

#ifdef __KERNEL__

/* thread startup structure */
struct ct_thread_startup
{
	struct completion *completion;	/* signalling init of new thread */
	int (*function)(void *);	/* function to run in the new thread */
};

#endif /* _KERNEL_ */

#endif

