#ifndef _CT_SYNC_PROTO_H
#define _CT_SYNC_PROTO_H

/* no need for a public declaration of cts_buff structure */
struct cts_buff;
struct cts_protoh;

/* protocol intialization functions */
struct cts_protoh *cts_proto_init(char *devname, 
				  const struct sockaddr_in *addr, 
				  u8 nodeid, int master,
				  void (*became_slave)(struct cts_protoh*,
					  		void *),
				  void *became_slave_arg,
				  wait_queue_head_t *wait);

/* protocol cleanup funciton */
void cts_proto_cleanup(struct cts_protoh *cph);

/* is protocol state currently 'master' ? */
int cts_proto_is_master(struct cts_protoh *cph);

/* tell protocol to make local node master */
int cts_proto_become_master(struct cts_protoh *cph);

/* signal that an initsync is to follow */
int cts_proto_initsync_follows(struct cts_protoh *cph);

/* request initial re-sync from master */
int cts_proto_request_initsync(struct cts_protoh *cph);

/* SENDER */

/* advance current buffer of send ring if it's old enough,
 * so that it will be sent even if it's not full */
int cts_proto_advance_timeout(struct cts_protoh *cph);

/* Sender wants to enqueue 'size' bytes. Returns pointer to buffer */
void *cts_proto_want_enqueue(struct cts_protoh *cph, 
			     struct cts_buff **pcsb, unsigned int size);
/* after filling in the data, csb_use_dec has to be called on (*pcsb) */
void csb_use_dec(struct cts_protoh *cph, struct cts_buff *csb);

int cts_proto_send_pending(struct cts_protoh *cph);
/* dequeue a to-be-send cts_buff, otherwise put current task to sleep */
int cts_proto_send_dequeue(struct cts_protoh *cph);



/* RECEIVER */

/* possible return values of cts_proto_rcv_pkt() */
#define CTS_PROTO_RCV_ERROR		-1
#define CTS_PROTO_RCV_DISCARDED		0
#define CTS_PROTO_RCV_SYNC		1
#define CTS_PROTO_RCV_NACK		2
#define CTS_PROTO_RCV_MASTERANN		3
#define CTS_PROTO_RCV_INITSYNC		4

/* check if we have pending packets in the receive buffer */
int cts_proto_recv_pending(struct cts_protoh *cph);
/* receive a packet, and enqueue it in the ring buffer if needed */
int cts_proto_recv_pkt(struct cts_protoh *cph);

void cts_proto_recv_running(struct cts_protoh *cph);

/* get next message out of packet receiver queue */
struct ct_sync_msghdr *cts_proto_recvmsg(struct cts_protoh *cph, struct ct_sync_pkthdr **hdr);

#endif /* _CT_SYNC_PROTO_H */
